#include <stdlib.h>
#include <iostream>
#include "mystd.h"
#include "myexception.h"

#include <readline/readline.h>
#include <readline/history.h>

using namespace std; 

#include "asi.h"
#include "gar.h"

// *** decode routines ***

char bask_decode(const char *resp, bool *rval, const CAskPars *AP)
{
	if (resp[0]=='\0') {
		// printf("understood default\n");
		return 'd';
	} else if (resp[1]=='\0') {
		if (strspn(resp, "y1")) {
			// printf("understood y\n");
			*rval = 1;
			return 'v';
		}
		else if (strspn(resp, "n0")) {
			// printf("understood n\n");
			*rval = 0;
			return 'v';
		}
	}
	return 'h';
}

char iask_decode(const char *resp, int *rval, const CAskPars *AP)
{
        int rexp, i;
	if (resp[0]=='\0') {
		return 'd';
	} else if (( sscanf(resp, "%de%d", rval, &rexp)==2 ||
                     sscanf(resp, "%dE%d", rval, &rexp)==2 ) &&
                     rexp>=0 ) {
                for ( i=0; i<rexp; ++i ) { *rval *= 10; }
		return 'v';
	} else if (sscanf(resp, "%d", rval)==1) {
		return 'v';
	}
	return 'h';
}

char dask_decode(const char *resp, double *rval, const CAskPars *AP)
{
	if (resp[0]=='\0') {
		return 'd';
	} 
	// else if (sscanf(resp, "%lg", rval)==1) {
	try{
            *rval = mystd::any2dbl( resp );
            return 'v';
	}
        catch(MyException &e) {
            return 'h';
        }
}

char cask_decode(const char *resp, char *rval, const CAskPars *AP)
{
	if (resp[0]=='\0') {
		return 'd';
	} else if (resp[1]=='\0') {
		*rval = resp[0];
		return 'v';
	}
	return 'h';
}

char sask_decode(const char *resp, string *rval, const CAskPars *AP)
{
	if (resp[0]=='\0') {
		return 'd';
	} else if (!strcmp(resp,"\\EOL")) {
		*rval = string("");
		return 'v';
	} else if (!strcmp(resp,"\\?")) {
		return 'h';
	} else {
		*rval = string(resp);
		return 'v';
	}
}

// *** intermediate calls for simple types: ***

const char *bask_helptext = "a boolean value (y or n)";
const char *iask_helptext = "an integer number";
const char *dask_helptext = "a floating-point number";
const char *cask_helptext = "one character";
const char *sask_helptext = "a string";

bool   bask(const char *quest, const int defflag, const bool& defval)
{
	return (bool) myask(string(quest), defflag | ASK_WORD, defval, 
			    bask_helptext, bask_decode, NULL, NULL);
}

int    iask(const char *quest, const int defflag, const int& defval)
{
	return (int) myask(string(quest), defflag | ASK_WORD, defval,
			   iask_helptext, iask_decode, NULL, NULL);
}

double dask(const char *quest, const int defflag, const double& defval)
{
	return (double) myask(string(quest), defflag | ASK_WORD, defval,
			      dask_helptext, dask_decode, NULL, NULL);
}

char   cask(const char *quest, const int defflag, const char& defval)
{
	return (char) myask(string(quest), defflag | ASK_WORD, defval, 
			    cask_helptext, cask_decode, NULL, NULL);
}

string wask(const char *quest, const int defflag, const string& defval)
{
	return (string) myask(string(quest), defflag | ASK_WORD, defval,
			      sask_helptext, sask_decode, NULL, NULL);
} // ACHTUNG: umstellung auf wask_ unfertig :ACHTUNG

string sask(const char *quest, const int defflag, const string& defval)
{
	return (string) myask(string(quest), defflag, defval, 
			      sask_helptext, sask_decode, NULL, NULL);
}

// *** calls with constraints: ***

/*
int    iask_lim(const char *quest, const class CLimits& Lim)
{ 
	return iask_lim(quest, 0, 0, Lim); 
}

int    iask_lim(const char *quest, const int& v, const class CLimits& Lim)
{ 
	return iask_lim(quest, 1, v, Lim); 
}

double dask_rge(const char *quest, const class CRange& Rge)
{ 
	return dask_rge(quest, 0, 0, Rge); 
}

double dask_rge(const char *quest, const double& v, const class CRange& Rge)
{ 
	return dask_rge(quest, 1, v, Rge);
}

bool   iask_lim_check (const void *val, const void *args)
{
	return ((CLimits *)args)->contained(*((int *)val));
}

int    iask_lim(const char *quest, int defflag, const int& defval, 
		const class CLimits& Lim)
{
	string s;  
	s = "value must be within limits " + Lim.str();
	
	VConstraints VC;
	VC.push_back(CConstraints(iask_lim_check, (const void *) &Lim, s));

	return (int) myask(string(quest), defflag, defval,
			   iask_helptext, iask_decode, NULL, &VC);
}

bool   dask_lim_check (const void *val, const void *args)
{
	return ((CLimits *)args)->contained(*((double *)val));
}

double dask_rge(const char *quest, int defflag, const double& defval, 
		const CRange& Rge)
{
	string s;  
	s = "value must be in range " + Rge.str();
	VConstraints VC;
	VC.push_back(CConstraints(dask_lim_check, (const void *) &Rge, s));
	return (double) myask(string(quest), defflag, defval,
			      dask_helptext, dask_decode, NULL, &VC);
}
*/

#ifdef _ALTASI
char oask_decode(const char *resp, int *rval, const CAskPars *AP)
{
	if (resp[0]=='\0')
		return 'd';
	char **options;
	options = (char **) AP;
	for (int n=0; ; n++) {
		if (strlen(options[2*n])==0) break; // no more options
		if (!strcmp(resp,options[2*n])) {
			*rval = n+1;
			return 'v';
		}
	}
	if (!strcmp(resp,"q")) {
		*rval = 0;
		return 'v';
	} else 
		return 'h';
}
#endif
