/* xax: eXtended Array eXpressions */

/* developed with GNU Bison 1.35 */

%{
using namespace std;
#include <math.h>  /* For math functions, cos(), sin(), etc. */
#include <string.h> /* for verbose error in C++ */
#include <stdlib.h> /* for verbose error in C++ */
// #include "func.h"
#include "node.h"
#include "myexception.h"
%}

%pure-parser
%debug

%{
#define YYSTYPE NodePtr
/* #define YYERROR_VERBOSE 1 /* redundant with %debug ? */
#define YYPARSE_PARAM pA     /* xaxparse() becomes xaxparse(void*) */
int xaxerror(char*);         /* provided by calling code (expr.cpp, ...) */
int xaxlex(YYSTYPE *xaxlval); /* created by lex.l */
%}

%token   WS 
%token   NUM STRG
%token   REF FRF FNCT END
%token   BOP_EQ BOP_NE BOP_GT BOP_GE BOP_LT BOP_LE BOP_AND BOP_OR BOP_XOR BOP_OVER
%token   BOP_IN BOP_RGE

/* Operator precedence */

%left '?' ':'
%left BOP_OR
%left BOP_AND
%left BOP_XOR
%left BOP_EQ BOP_NE
%left BOP_RNG
%left BOP_GT BOP_GE BOP_LT BOP_LE BOP_IN
%left '-' '+'
%left NEG     /* arithmetic negation (unary minus) */
%left '*' '/' '%' BOP_MOD
%left BOP_OVER /* binomial coefficient */
%right '!'    /* logical not */
%right '^'    /* exponentiation */

/* Grammar follows */

%%
input: 
        /* empty */         { throw BadInput( "empty line" ); }
      | line
;

line:
        obj END             { *((NodePtr*)pA) = $1; return 0; }
      | error               { yyerrok; return -1; }
;

obj:                       /* objects of different types */
        log                 { $$ = $1; }
/*
      | rng                 { $$ = $1; }
*/
;

/*
rng:
        sum BOP_RGE sum     { $$ = NodePtr( new NodeRange( $1, $3) ); }
;
*/

log:                       /* logical expressions */
        sum                 { $$ = $1; }
/*
      | sum BOP_IN  rng     { $$ = NodePtr( new NodeInside( $1, $3) ); }
      | log '?' log ':' log { $$ = new CTree(NFunctions::find("?:"), $1, $3, $5); }
      | log BOP_EQ  log     { $$ = new CTree(NFunctions::find("=="),  $1, $3); }
      | log BOP_NE  log     { $$ = new CTree(NFunctions::find("!="),  $1, $3); }
      | log BOP_GT  log     { $$ = new CTree(NFunctions::find(">"),  $1, $3); }
      | log BOP_GE  log     { $$ = new CTree(NFunctions::find(">="),  $1, $3); }
      | log BOP_LT  log     { $$ = new CTree(NFunctions::find("<="),  $1, $3); }
      | log BOP_LE  log     { $$ = new CTree(NFunctions::find("<"),  $1, $3); }
      | log BOP_AND log     { $$ = new CTree(NFunctions::find("&&"), $1, $3); }
      | log BOP_OR  log     { $$ = new CTree(NFunctions::find("||"),  $1, $3); }
      | log BOP_XOR log     { $$ = new CTree(NFunctions::find("xor"), $1, $3); }
*/
;

sum:                       /* a sum (unary +/- only allowed in first term) */
        neg                 { $$ = $1; }
      | neg '+' asu         { $$ = NodePtr( new NodeBinop( $1, $3, "+") ); }
      | neg '-' asu         { $$ = NodePtr( new NodeBinop( $1, $3, "-") ); }
;

asu:                       /* sum of terms without unary +/- prefix */
      prd                   { $$ = $1; }
      | prd '+' prd         { $$ = NodePtr( new NodeBinop( $1, $3, "+") ); }
      | prd '-' prd         { $$ = NodePtr( new NodeBinop( $1, $3, "-") ); }
;

neg:                       /* product term with optional +/- prefix */
        prd                 { $$ = $1; }
/*
      | '+' prd  %prec NEG  { $$ = $2; }
      | '-' prd  %prec NEG  { $$ = NodePtr( new NodeNeg( $2 )); }
*/
;

prd:                       /* products and other terms that may be preceded by +/- */
        NUM                 { $$ = $1; }
      | STRG                { $$ = $1; }
      | FNCT '(' log ')'    { $$ = $1; $1->SetArg( $3 ); }
/*
      | '(' log ')'         { $$ = NodePtr( new NodeParenthesis( $2 )); } 
      | FNCT '(' log ',' log ')' { $$ = new CTree((CFunc*) $1, $3, $5 ); }
      | prd '*' prd        { $$ = NodePtr( new NodeMul( $1, $3) ); }
      | prd '/' prd        { $$ = NodePtr( new NodeDiv( $1, $3) ); }
      | prd '%' prd        { $$ = new CTree(NFunctions::find("%"), $1, $3); }
      | prd BOP_MOD prd    { $$ = new CTree(NFunctions::find("mod"), $1, $3); }
      | prd BOP_OVER prd   { $$ = new CTree(NFunctions::find("over"), $1, $3); }
      | prd '^' prd        { $$ = new CTree(NFunctions::find("^"), $1, $3); }
      | '!' prd            { $$ = new CTree(NFunctions::find("!"), $2); }
*/
;

/*
arr:
      '[' list ']'         { $$ = $2; }
;
*/
/*
list:
        log                { $$ = new vector<CTree*>; $$->push_back($1); }
      | list ',' log       { $$ = $1; $$->push_back($3); }
;
*/

/*iva:                       /* indexed variable */
/*
        REF                { $$ = new CRef(string((char*)$1)); }
      | REF '[' rex ']'    { $$ = new CRef(string((char*)$1), $3); }
      | REF '[' rex ',' rex ']'   
                           { $$ = new CRef(string((char*)$1), $3, $5); }
      | REF '[' rex ',' rex ',' rex ']'   
                           { $$ = new CRef(string((char*)$1), $3, $5, $7); }
;
*/

/* ifu:                       /* indexed function */
/*
        FRF '[' rex ']'    { $$ = new CRef(string((char*)$1), $3); }
      | FRF '[' rex ',' rex ']'   
                           { $$ = new CRef(string((char*)$1), $3, $5); }
;
*/

rex:                       /* reference expression */
        /* empty */        { $$ = 0;}
      | log                { $$ = $1; }
;

/* End of grammar */
%%
