// this is gar.h: the generic ask routine

#include <vector>
#include <iostream>
using namespace std;

#include "readln.h"

// *** user-imposed constraints and parameters: ***

class CConstraints {
 public:
	bool (*check)(const void *, const void *);
	const void *args;
	string text;
	
	CConstraints(bool (*check_in)(const void *, const void *),
		     const void *args_in, const string& text_in);
};

typedef vector<CConstraints> VConstraints;

typedef void CAskPars;

// *** inline implementation: ***

#define ASK_DEF_GIVEN     1
#define ASK_DEF_NOT_SHOW  2
#define ASK_WORD          4

template <class T>
T myask(const string quest, const int flags, 
	const T& defval, const string helptext, 
	char decode(const char *resp2, T *rv2, const CAskPars *AP2),
	const CAskPars *AP, VConstraints *VC)
{
	string prompt, resp;
	char test;
	T rval;

	// check default:
	bool defvalid = flags & ASK_DEF_GIVEN;
	if (defvalid && VC) {
		for (VConstraints::iterator cc=VC->begin(); cc<VC->end(); ++cc){
			if (!cc->check((const void*) &defval, cc->args)) {
				// printf("DEBUG: default invalidated\n");
				// cout << "-> constraint: "<<cc->text<<endl;
				defvalid = 0;
				break;
			}
		}
	}

	prompt = quest;
	if (defvalid && !(flags & ASK_DEF_NOT_SHOW))
		prompt += " [" + strg(defval) + "] ? ";
	else
		prompt += " ";

	bool ok; 
	do {
		if (flags & ASK_WORD)
			resp = NRead::readwd(prompt);
		else
			resp = NRead::readln(prompt);
		test = decode(resp.c_str(), &rval, AP);
		ok = 1;
		switch (test) {
		case 'v':
			break;
		case 'd':
			if (defvalid) {
				rval = defval;
				break;
			} 
			cout << "no default available\n";
		case 'h':
		default:
			cout << "-> please enter " << helptext;
			if (defvalid)
				cout << ";\n"
			 "-> just enter RETURN to take the default ["
				     << strg(defval) << "]";
			cout << ".\n";
			ok = 0;
		}
		if (VC) {
			for (VConstraints::iterator cc=VC->begin(); 
			     cc<VC->end(); ++cc) {
				if (cc->check(&rval, cc->args)) {
					// cout << "constraint fulfilled \n";
				} else {
					cout << "-> constraint: " 
					     << cc->text << endl;
					ok = 0;
					continue;
				}
			}
		}
	} while (!ok);
	return rval;
}
