#include "mystd.h"
#include "range.h"
#include "index.h"
#include "list.h"
#include "coord.h"
#include "grid.h"
#include "scan.h"
// #include "curve.h"
// #include "func.h"
// #include "expr.h"

class COlo { // on-line object (virtual base class for COld, COlc)
 public:
	string name;
	vector<string> lDoc;
	CCoord xco, yco;
	vector<CCoord> ZCo;
	vector<CParam> RPar;
        bool as_on_disk;

	virtual inline ~COlo(void) { };
	inline uint nZ(void) const { return ZCo.size(); };
	int SaveHeader(FILE *fd);
};

class COld : public COlo { // on-line data file
 public:
	vector<CScan> VS;

	inline COld(void) { };
//	COld(class COlc *c);
	COld(CCoord CX, CCoord CY, CGrid VX, vector<double> VY);
	inline ~COld(void) { };

	int SaveBody( FILE *fd, string fmt );

	inline uint nScan(void) const { return VS.size(); };
	uint nPts(uint j) const; 
	uint nPts(void) const; // 0 unless all spectra j have same nPts(j)
};

/*
class COlc : public COlo { // on-line curves
 public:
	string expr;
        CTree T;
	
	uint no_dfil;
	vector<CCoord> PCo;
        vector<int>    Fixed; // should be vector<bool>, but trouble with IListV
	vector<CCurve> VC;

        bool weight_y_log;

	inline COlc(void) { };
	inline COlc(COld *d) : COlo((COlo)*d) { };
	inline ~COlc(void) { };

	int SaveBody(FILE *fd);

	inline uint nScan(void) const { return VC.size(); };
	inline uint nPar(void) const { return PCo.size(); };
};
*/
// the following is needed to pack (pointers to) COld and COlc into ONE vector :
class CEle {
	enum eleTyp { DATA, CURVE }; 
	eleTyp Typ;
	void* Ptr;

 public:
	inline CEle(void) { };
	inline CEle(COld* pin) {
		Typ = DATA;
		Ptr = (void*) pin;
	};
//	inline CEle(COlc* pin) {
//		Typ = CURVE;
//		Ptr = (void*) pin;
//	};
	// inline ~CEle(void) {  };
	
        inline bool isD(void) { return Typ==DATA; };

	COld* D(void) const {
		if (Typ==DATA)  return (COld*) Ptr;
		else            return 0;
	};
// 	COlc* C(void) const {
// 		if (Typ==CURVE) return (COlc*) Ptr;
// 		else            return 0;
// 	};
	COlo* P(void) const { return (COlo*) Ptr; };

	void predelete(void) { 
		if      (Typ==DATA)  delete (COld*) Ptr;
//		else if (Typ==CURVE) delete (COlc*) Ptr;
		else exit(7);
	};

  	uint nScan(void) const {
		if      (Typ==DATA)  return ((COld*) Ptr)->nScan();
//		else if (Typ==CURVE) return ((COlc*) Ptr)->nScan();
		else exit(7);
	};
	uint nPts(void) const {
		if      (Typ==DATA)  return ((COld*) Ptr)->nPts();
//		else if (Typ==CURVE) return ((COlc*) Ptr)->PCo.size();
		else exit(7);
	};
	vector<double> *Z(uint j) const {
		if      (Typ==DATA)  return &(((COld*) Ptr)->VS[j].z);
//		else if (Typ==CURVE) return &(((COlc*) Ptr)->VC[j].z);
		else exit(7);
	};
	void Z2R(void);
	int Load( FILE *fd, string fshort );
	int Save( FILE *fd, string fmt );
};
		
	
namespace NOlm { // on-line memory
        // the data:
        extern vector<CEle> MOM; // the mother of all on-line data

        // the current selection:
        extern CList FSel;
        extern CList FSelOld;
        extern bool overwrite;

  	// modify data:
	void SelNew(void);
	void SelPop(void);
        int OloDup( COlo *olo_in, CEle *ele_out);
        CEle* OloPushBack( CEle *e );
	CEle* OloAdd(class COlo *olo);
	void OlfDel(void);
	void OlfCopy(void);
	void OlfLoop(int (*exec) (int fno, COld *fin, COld *fout));

        // the current scan selection:
        int FJSelAsk (string quest);
        void FJSelGet (uint fno, class CList *JSel);
        void FJSelGetNoask (uint fno, class CList *JSel);

	class Iterate { // iterator over FSel (virtual base)
	protected:
		uint iV;
		CList Sel;
	public:
		inline Iterate(void) { Sel = FSel; Reset(); };
		inline void Reset(void) { iV = 0; };
		inline void All(void) { Sel = CList(0,MOM.size()-1); };
		inline uint SelNo(void) const { return Sel.V[iV-1]; }
		inline uint index(void) const { return iV-1; }
		inline uint size(void) const { return Sel.V.size(); }
	};
	class IterateEle : public Iterate { // all types
	public:
		inline CEle* operator()(void) {
			if (iV>=Sel.n()) return 0;
			return &(MOM[Sel.V[iV++]]);
		};
	};
	class IterateD : public Iterate { // must be data file
		COld *e;
	public:
		inline COld* operator()(void) {
			do {
				if (iV>=Sel.n()) return 0;
			} while (! (e = MOM[Sel.V[iV++]].D()));
			// cout << "iter -> file no " << iV-1 << "\n";
			return e;
		};
	};
/*
	class IterateC : public Iterate { // must be curve file
		COlc *e;
	public:
		inline COlc* operator()(void) {
			do {
				if (iV>=Sel.n()) return 0;
			} while (! (e = MOM[Sel.V[iV++]].C()));
			// cout << "iter -> file no " << iV-1 << "\n";
			return e;
		};
	};
*/
};
