// This is list.h
//

#include <vector>

class CList {
 private:
	bool synval, limneeded;
	CLimits Lims;
	string instruction;

 public:
	vector<uint> V;

	// initialize by instruction:
	CList(string inp="-", class CLimits Lims_in=CLimits(1));
	inline CList(string inp, uint ni_in, uint nf_in)
		{ *this = CList(inp, CLimits(ni_in, nf_in)); };

	// initialize by values:
	CList(uint val);
	CList(uint ni, uint nf);
	inline CList(CLimits limin) { *this = CList(limin.inf, limin.sup); };

	inline bool synvalid() const { return synval; };
	inline bool semvalid() const { return synvalid() && 
					       !(limneeded && !limgiven()); };
	inline uint n() const { return semvalid() ? V.size() : 0 ; };
	bool contained(uint val) const;

	CList Ask(const char* quest);
	CList Ask(const char* quest, class CLimits Lims);
	CList Ask(const char* quest, CList Def);
	CList Ask(const char* quest, CList Def, class CLimits Lims);
	CList Ask(const char* quest, bool defflag, CList Def, 
		  class CLimits Lims);

	void ProvideLims(class CLimits Lims_in);
	inline void ProvideLims(uint ni, uint nf) 
		{ ProvideLims(CLimits(ni, nf)); };

	void Sort(void);
	void SortReverse(void);

	CList operator+(const CList L) const;
	CList operator+(const uint val) const;
	CList operator+=(const CList L);
	CList operator+=(const uint val);

	string str(void) const;
	inline const char* c_str(void) const
		{ return str().c_str(); };
	inline friend ostream& operator<< (ostream& s, CList L) 
		{ return s << L.str(); };

 private:
	inline bool limgiven() const { return Lims.finite(); };

	void Parse(string inp);
	void ParseList(string inp);
	void ParseSkipinterval(string inp);
	int  ParseSkipfactor(string inp);
	void ParseInterval(string inp, int skip);
	int  ParseListvalue(string inp, int which);

	string sel2text(void) const;

	static const char *helptext;
};	

inline string strg(const CList& L) { return L.str(); };

template<class T>
class IListV { // iterator over vector elements selected as a list
	CList Selection;
	uint iV; // vornehmer waere: vector<uint>::iterator 
	vector<T> *TV;
 public:
	IListV<T>(vector<T> *TV_in, CList *Sel_in) {
		TV = TV_in;
		Selection = *Sel_in;
		iV = 0;
	};
	T* operator()(void) {
		if (iV<Selection.n())
			return &((*TV)[Selection.V[iV++]]);
		else
			return 0;
	};
	uint SelNo(void) { return Selection.V[iV-1]; };
};

class ListIter { 
 private:
	uint i;
	CList L;
 public:
	inline ListIter(void) { Reset(); };
	inline ListIter(CList L_in) { Reset(L_in); };
	inline void Reset(void) { i=0; };
	inline void Reset(CList L_in) { L = L_in; Reset(); };
	inline uint operator()(uint *val) {
		if (i>=L.n()) return 0;
		*val = L.V[i++];
		return i;
	};
};

